import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:medapay/entity.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:webview_flutter/webview_flutter.dart';

class MedaPayLauncher {
  String _apiKey;
  bool _isSandbox;

  MedaPayLauncher({@required String apiKey, @required bool isSandbox}) {
    _apiKey = apiKey;
    _isSandbox = isSandbox;
  }

  String _buildUrl(MedaPayPayment payment) {
    String url = 'https://sdk.pay.meda.chat/';
    if (_isSandbox) url = 'https://sdkuat.pay.meda.chat/';

    url += '?_a=' + _encode(payment);

    return url;
  }

  String _encode(MedaPayPayment payment) {
    String json = jsonEncode(payment);
    var bytes = utf8.encode(json);
    String base64String = base64Encode(bytes);
    String cleanedString = base64String.replaceAll("=", "");
    return cleanedString.substring(5) + cleanedString.substring(0, 5);
  }

  Widget _getMedaPayWebview(BuildContext context, MedaPayPayment payment,
      Function onReferenceNumber, Function onSuccess, Function onError) {
    return WebView(
        javascriptMode: JavascriptMode.unrestricted,
        gestureNavigationEnabled: true,
        gestureRecognizers: <Factory<OneSequenceGestureRecognizer>>{
          Factory<VerticalDragGestureRecognizer>(
            () => VerticalDragGestureRecognizer(),
          ),
        },
        javascriptChannels: {
          JavascriptChannel(
              name: 'MedaPay',
              onMessageReceived: (JavascriptMessage message) {
                dynamic theCommand = jsonDecode(message.message);
                if (theCommand['command'] == 'closeWindow') {
                  Navigator.pop(context);
                }
                if (theCommand['command'] == 'onReferenceNumber') {
                  onReferenceNumber(theCommand['data']);
                }

                if (theCommand['command'] == 'onSuccess') {
                  onSuccess(theCommand['data']);
                }

                if (theCommand['command'] == 'initCall') {
                  launch('tel://' + theCommand['data']);
                }
              }),
        },
        onWebViewCreated: (WebViewController webViewController) {
          Map<String, String> headers = {"x-api-key": _apiKey};
          webViewController.loadUrl(_buildUrl(payment), headers: headers);
        }
//        onPageStarted: (url) {
//          print("loading page:" + url);
//        },
//        onPageFinished: (url) {
//          print("page loaded:" + url);
//        },
//        onWebResourceError: (error) {
//          print(error);
//        }
        );
  }

  void init(
      {@required BuildContext context,
      @required MedaPayPayment payment,
      @required Function onReferenceNumber,
      @required Function onSuccess,
      @required Function onError,
      Function onClose}) {
    showModalBottomSheet(
        context: context,
        isDismissible: false,
        builder: (BuildContext bc) {
          return _getMedaPayWebview(
              bc, payment, onReferenceNumber, onSuccess, onError);
        });
  }
}
